function pad(c) {
	var i = parseInt(c, 10);
	return (i < 10 ? '0' + i : c.toString());
}

function configureAction() {
	if (raisedEvent.parameter.startTime==null) {
		lWarn.logln('heating-controller.valve-protection configureAction:startTime is missing');
		return -1;
	}
	if (raisedEvent.parameter.startTime==null) {
		lWarn.logln('heating-controller.valve-protection configureAction:duration is missing');
		return -1;
	}
	storeParameter('generalSettings/valveProtectionEnabled',true);
	switch(Math.floor(raisedEvent.parameter.startTime/(24*60*60))) {
		case 0:
			storeParameter('generalSettings/valveProtectionWeekday','MO');
		break;
		case 1:
			storeParameter('generalSettings/valveProtectionWeekday','TU');
		break;
		case 2:
			storeParameter('generalSettings/valveProtectionWeekday','WE');
		break;
		case 3:
			storeParameter('generalSettings/valveProtectionWeekday','TH');
		break;
		case 4:
			storeParameter('generalSettings/valveProtectionWeekday','FR');
		break;
		case 5:
			storeParameter('generalSettings/valveProtectionWeekday','SA');
		break;
		case 6:
			storeParameter('generalSettings/valveProtectionWeekday','SU');
		break;
	}
	storeParameter('generalSettings/valveProtectionStartTime',Math.floor(raisedEvent.parameter.startTime%(24*60*60)));
	storeParameter('generalSettings/valueProtectionTimeFrame',parseInt(raisedEvent.parameter.duration,10));
	return 0;
}

function executeOneStep() {
	
	lInfo.logln('heating-controller.valve-protection queue ' + unescape(raisedEvent.parameter.queue) + " for index " + raisedEvent.parameter.index);
	var aParameter = JSON.parse(raisedEvent.parameter.queue);
	var iIndex = parseInt("" + raisedEvent.parameter.index, 10);
	if (iIndex < aParameter.length) {
		var zone = aParameter[iIndex];
		try {
			lInfo.logln('heating-controller.valve-protection zid ' + zone);
			var pZone = getZoneByID(zone);
			pZone.callScene(48, 31);
		} catch (e) {
			lWarn.logln(e);
		}
		new Event('heating-controller.valve-protection', {
			actions : 'queue',
			queue : raisedEvent.parameter.queue,
			index : (iIndex + 1),
			deltaTime: raisedEvent.parameter.deltaTime,
			time : '+' + raisedEvent.parameter.deltaTime
		}).raise();
	} else {
		lInfo.logln('heating-controller.valve-protection.queue finished');
			new Event('heating-controller.answer', {
				'requestingEvent' : 'heating-controller.valve-protection',
				'requestingAction' : 'queue',
				'result' : 0
			}).raise();
	}
}

function queueAction() {
	if (Property.getProperty('generalSettings/valveProtectionEnabled')==false) {
		lInfo.logln('valve protection is disabled, so skipping it now');
		return;
	}
	var deltaTime=Property.getProperty('generalSettings/valueProtectionTimeFrame');
	
	var oDevices = Property.getNode('/apartment/zones/zone0/devices').getChildren();
	var oZones=[];
	for (var iIndex=0;iIndex<oDevices.length;iIndex++) {
		if ((oDevices[iIndex].getChild('functionID').getValue()&0x3000)==0x3000)
			if (oDevices[iIndex].getChild('outputMode').getValue()>0){
				var zoneID=oDevices[iIndex].getChild('ZoneID').getValue();
				var fFound=false;
				for (var jIndex=0;jIndex<oZones.length;jIndex++)
					if (zoneID==oZones[jIndex])
						fFound=true;
				if (!fFound)
					oZones.push(zoneID);
			}
	}
	lInfo.logln('queue now valve protection for ' + JSON.stringify(oZones));
	
	new Event('heating-controller.valve-protection', {
		actions : 'queue',
		queue : JSON.stringify(oZones),
		index : 0,
		deltaTime: deltaTime,
		time : '+1'
	}).raise();
}


function scheduleAction() {
	if (Property.getNode('dynamic/scheduledValveSettings') != null) {
		var queueID = Property.getProperty('dynamic/scheduledValveSettings');
		var propNode = Property.getNode('/system/EventInterpreter/ScheduledEvents/' + queueID);
		if (propNode) {
			propNode.getParent().removeChild(propNode);
		}
		Property.getNode('dynamic').removeChild(Property.getNode('dynamic/scheduledValveSettings'));
	}
	
	var sRRule = "FREQ=WEEKLY;BYDAY=" + Property.getProperty('generalSettings/valveProtectionWeekday');

	var timeValue = Math.abs(Property.getProperty('generalSettings/valveProtectionStartTime'));
	var iTempStartTime;

	var dold = new Date();
	dold.setTime(dold.getTime() - (1000 * 60 * 60 * 24 * 7));
	
	iTempStartTime = new Date();
	iTempStartTime.setFullYear(dold.getFullYear());
	iTempStartTime.setMonth(dold.getMonth());
	iTempStartTime.setDate(dold.getDate());
	iTempStartTime.setHours(Math.floor(timeValue / 3600));
	iTempStartTime.setMinutes(Math.floor(timeValue / 60) % 60);
	iTempStartTime.setSeconds(timeValue % 60);
	iTempStartTime = iTempStartTime.getTime();

	var iCalStartTime = dold.getFullYear() + pad(dold.getMonth() + 1) + pad(dold.getDate()) + 'T' + pad(Math.floor(timeValue / 3600)) + pad(Math.floor(timeValue / 60) % 60)
			+ pad(timeValue % 60);

	timerEvent = new TimedICalEvent('heating-controller.valve-protection', iCalStartTime, sRRule, {'actions':'start'});
	Property.setProperty('dynamic/scheduledValveSettings', timerEvent.raise());	
}

if (raisedEvent.name=='heating-controller.valve-protection') {
	if (raisedEvent.parameter.actions!=null) {
		if (raisedEvent.parameter.actions=='schedule') {
			scheduleAction();
		}
		if (raisedEvent.parameter.actions=='configure') {
			var resultValue=configureAction();
			scheduleAction();
			Property.store();
			new Event('heating-controller.answer',{'requestingEvent':'heating-controller.valve-protection','requestingAction':'configure','result':resultValue}).raise();
		}
		if (raisedEvent.parameter.actions=='test') {
			queueAction();			
		}
		if (raisedEvent.parameter.actions=='start') {
			queueAction();			
		}
		if (raisedEvent.parameter.actions=='queue') {
			executeOneStep();			
		}
		if (raisedEvent.parameter.actions=='enable') {
			storeParameter('generalSettings/valveProtectionEnabled',true);
			Property.store();
			new Event('heating-controller.answer',{'requestingEvent':'heating-controller.valve-protection','requestingAction':'enable','result':0}).raise();
		}
		if (raisedEvent.parameter.actions=='disable') {
			storeParameter('generalSettings/valveProtectionEnabled',false);			
			Property.store();
			new Event('heating-controller.answer',{'requestingEvent':'heating-controller.valve-protection','requestingAction':'disable','result':0}).raise();
		}
	}
}